package altbase;

use strict;
#use warnings;

BEGIN {
    use Exporter   ();
    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

    # set the version for version checking
    $VERSION     = 1.00;

    @ISA         = qw(Exporter);
    @EXPORT      = qw(&new &init &parser &MakeDateTime &MakeDate &tagClean &tagConv &isInArray &specColorer &changeFormat &localeGrp &tagHardClean &messages &transPath);
    %EXPORT_TAGS = ( );
    @EXPORT_OK   = qw($path @month $linkstyle $errCode $errStr $header $redirect $maxId $maxPage $dsn $dbUser $dbPass $TTL);
}

our @EXPORT_OK;
# exported package globals go here
our @month	= () ;
our @totalGroups = ();
our %messages = ();
our $linkstyle	= undef;
our $errCode	= 0;
our $errStr	= undef;
our $header	= "Content-type: text/html\n\n";
our $redirect	= "Status: 302 Moved\nLocation: ";
our $maxId	= 100000;
our $maxPage	= 1000;
our $path	= "";
our $TTL = 3600*2; # Time to user stay logged in

#> MySQL
our $db     = "prometeus";       # Name of database
our $dbUser = "user";      # Name of MySQL user
our $dbPass = "password";    # User Password
my  $dbHost = "localhost"; # Host of MySQL
our $dsn    = "DBI:mysql:$db:$dbHost";	# just $dsn, no reason to change value

my %HTML = ();

my $lang = "ru1" ;
my $htf_root  = "/usr/share/prometeus/web/html/htf" ;
my %modules = (
                "inform"   => "$htf_root/inform.htf.$lang",
                "maintain" => "$htf_root/maintain.htf.$lang"
);

END { }


# new class altbase
sub new {
	my $self = shift;
	$self = "altbase" if (! defined($self));
	my $class = ref($self) || $self;
	return bless {},$class;
}

# Check if defined class altbase
sub self_or_default {
	return @_ if (defined($_[0]) && (ref($_[0]) eq "altbase"));
	my $Q = new altbase;
	return $Q,@_;
}

# init HTML hash
sub init {
	my $name = "";
    %HTML = ();
	my ($class,%Param) = self_or_default(@_);

	if ( exists $modules{$Param{'module'}} &&
	    open(MOD,"< $modules{$Param{'module'}}") ){
        while (<MOD>){
			if (/^\#/) { next; }
			if (/^==\{GLOBAL\}==/){
				while(<MOD>){
					if (/^\#/) { next; }
					if (/^==\{GLOBAL\}==/){ last; }
					else {
						if (/^\[(\w+)\]$/){
						$name=$1;
							while (<MOD>){
								if (/^\[\/$name\]$/) { chomp($HTML{$name}); last; }
								else { $HTML{$name}.=$_;}
							}
						}
					}
				}
			}
			if (/^==\{$Param{'script'}\}==/){
				while(<MOD>){
					if (/^\#/) { next; }
					if (/^==\{$Param{'script'}\}==/){ last; }
					else {
						if (/^\[(\w+)\]$/){
							$name=$1;
							while (<MOD>){
								if (/^\[\/$name\]$/) { chomp($HTML{$name}); last; }
								else { $HTML{$name}.=$_;}
							}
						}
					}
				}
				last;
			}
		}
		close(MOD);
		if ( defined($HTML{"MONTH"}) ){
			@month = split(/\s/,$HTML{"MONTH"});
			delete $HTML{"MONTH"};
			$linkstyle = $HTML{"AHREF"};
			delete $HTML{"AHREF"};
		}
        if ( defined($HTML{"GROUPS"}) ){
            @totalGroups = split(/\n/,$HTML{"GROUPS"});
            delete $HTML{"GROUPS"};
        }
	if ( defined($HTML{"MESSAGES"}) ){
            %messages = split(/\n/,$HTML{"MESSAGES"});
            delete $HTML{"MESSAGES"};
	}
	} else {
		$errCode = 1;
		$errStr = "not found HTML-codes for this module";
		return undef;
	}
	return 1;
}

# parse HTML-code & replace variables
sub parser {
	my ($class,%Param) = self_or_default(@_);
	my $text = $HTML{$Param{"HTML"}};
	my %vars = %{$Param{"vars"}};
	my %iflist = %{$Param{"iflist"}};

	$text =~ s/\{if\s(\w+)\}(.+?)\{endif\}/($iflist{$1} != 0 || $iflist{$1} eq "yes")?$2:''/sge if (keys %iflist);
	$text =~ s/\{if\s\!(\w+)\}(.+?)\{endif\}/($iflist{$1} == 0 || $iflist{$1} eq "no")?$2:''/sge if (keys %iflist);
	$text =~ s/%(\w+)%/$vars{$1}/g if (keys %vars);
	return $text;
}

sub tagClean {
	my ($class,$text) = self_or_default(@_);
	if (ref($text)){
		${$text} =~ s/>/&gt;/g;
		${$text} =~ s/</&lt;/g;
		${$text} =~ s/"/&quot;/g;
		${$text} =~ s/\n/<br>/g;
		${$text} =~ s/\r//g;
		return 0;
	} else {
		$text =~ s/>/&gt;/g;
		$text =~ s/</&lt;/g;
		$text =~ s/"/&quot;/g;
		$text =~ s/\n/<br>/g;
		$text =~ s/\r//g;
		return $text;
	}
}

sub tagHardClean {
	my ($class,$text) = self_or_default(@_);
	if (ref($text)){
		${$text} =~ s/>/&gt;/g;
		${$text} =~ s/</&lt;/g;
		${$text} =~ s/"/&quot;/g;
		${$text} =~ s/\n/ /g;
		${$text} =~ s/\r//g;
		return 0;
	} else {
		$text =~ s/>/&gt;/g;
		$text =~ s/</&lt;/g;
		$text =~ s/"/&quot;/g;
		$text =~ s/\n/ /g;
		$text =~ s/\r//g;
		return $text;
	}
}

sub changeFormat {
	my ($class,$text) = self_or_default(@_);
	if (ref($text)){
		${$text} =~ s/>/&gt;/g;
		${$text} =~ s/</&lt;/g;
		${$text} =~ s/"/&quot;/g;
		${$text} =~ s/^(\s*)\-\s/$1<li>/g;
		${$text} =~ s/(\n\s*)\-\s/$1<li>/g;
		${$text} =~ s/\x20\x20/\x20&nbsp;/g;
    	${$text} =~ s/\n/<br>/g;
		${$text} =~ s/\r//g;
		return 0;
	} else {
		$text =~ s/>/&gt;/g;
		$text =~ s/</&lt;/g;
		$text =~ s/"/&quot;/g;
		$text =~ s/^(\s*)\-\s/$1<li>/g;
		$text =~ s/(\n\s*)\-\s/$1<li>/g;
		$text =~ s/\x20\x20/\x20&nbsp;/g;
		$text =~ s/\n/<br>/g;
		$text =~ s/\r//g;
		return $text;
	}
}

sub specColorer{
	my ($class,$text) = self_or_default(@_);
	if (ref($text)){
		${$text}=~ s/>/&gt;/g;
		${$text}=~ s/</&lt;/g;
		${$text}=~ s/\"/&quot;/g;
		${$text}=~ s/\@/ at /g;
		${$text}=~ s/(Url\:\s+)(((http|ftp)\:\/\/[^\s\[\]]{1,100})([^\s\]\[]*))/"$1".&linkme($2,$3,$5)/gie;
		${$text}=~ s/\x20\x20/\x20&nbsp;/g;
        ${$text}=~ s/^(\w+\:)/<b>$1<\/b>/g;
        ${$text}=~ s/^(\s*\#.*)/<b class='comment'>$1<\/b>/g;
        ${$text}=~ s/\n([\w\(\)\-\.]+\:)/\n<b>$1<\/b>/g;
        ${$text}=~ s/\n(\s*\#.*)/\n<b class='comment'>$1<\/b>/g;
        ${$text}=~ s/\n\%(description|prep|build|install|preun|pre|postun|post|triggerpostun|trigger|files|changelog|package)([\n|\s])/\n<b class='reserved'>\%$1<\/b>$2/g;
		${$text}=~ s/\n/<br>/g;
		${$text}=~ s/\r//g;
        return 0;
    } else {
        return undef;
    }
}

sub tagConv{
	my ($class,$text,$length) = self_or_default(@_);
    if (! defined($length) || $length < 5) {
        $length = 40;
    }

	if (ref($text)){
		${$text}=~ s/>/&gt;/g;
		${$text}=~ s/</&lt;/g;
		${$text}=~ s/\"/&quot;/g;
		${$text}=~ s/(((https?|ftp)\:\/\/[^\s\[\]]{1,$length})([^\s\]\[]*))/&linkme($1,$2,$4)/ge;
		${$text}=~ s/([\w\-\&\.]+\@[\w\-\&]+\.[\w\-\&\.]+)/<a href="mailto:$1" class="$linkstyle">$1<\/a>/g;
		${$text}=~ s/\[a\s(\<a[^\>]+?\>)[^\>]+?\>\]/$1/ig;
		${$text}=~ s/\[a\s([^\]]+?)\]/<a href="$1" class="$linkstyle">/ig;
		${$text}=~ s/\[\/a]/\<\/a\>/ig;
		${$text}=~ s/\[img\s+src\=\'?([^\']+?)\'?\]/\<IMG src=\"$1\"$2 vspace=\"4\" hspace=\"4\"\>/ig;
		${$text}=~ s/\[([\/]?)b\]/<$1b>/ig;
		${$text}=~ s/\[([\/]?)i\]/<$1i>/ig;
		${$text}=~ s/\[([\/]?)u\]/<$1u>/ig;
		${$text}=~ s/\[\&gt\;\&gt\;\]/<div align="right">/ig;
		${$text}=~ s/\[\&lt\;\&lt\;\]/<div align="left">/ig;
		${$text}=~ s/\[\&gt\;\&lt\;\]/<div align="center">/ig;
		${$text}=~ s/\[\&lt\;\&gt\;\]/<div align="justify">/ig;
		${$text}=~ s/\[\/\&[lgt\;].+?\;\]/<\/div>/ig;
		${$text}=~ s/\<\/div\>\n/\<\/div\>/g;
		${$text}=~ s/\[\*\]/&nbsp;&nbsp;<b>&middot;<\/b>&nbsp;/g;
		${$text}=~ s/\[\+\]/<span style="font-size:120%;">/g;
		${$text}=~ s/\[\-\]/<span style="font-size:80%;">/g;
		${$text}=~ s/\[\/[\+\-]\]/<\/span>/g;
		${$text}=~ s/\x20\x20/\x20&nbsp;/g;
		${$text}=~ s/\n/<br>/g;
		${$text}=~ s/\r//g;
		return 0;
	} else {
		return undef;
	}
}

sub linkme
{
	my $fulllink = shift;
	my $cutlink  = shift;
	my $endlink  = shift;
	$fulllink =~ s/([^\w\=\-\/]*)$//;
	my $erase = $1;

	my $test = qq|<a href="$fulllink" class="$linkstyle">|;
	if ($endlink eq "") {
		$test.=$fulllink."</a>$erase";
	} else {
		$test.=$cutlink."&hellip;</a>$erase";
	}

	return $test;
}

sub MakeDate{
	my ($class,$time) = self_or_default(@_);
	my @t = ($time =~ /^(\d+)-(\d+)-(\d+)/);
	$t[2] += 0;
	$time = $t[2]." ".$month[$t[1]-1]." ".$t[0];
	return $time;
}

sub MakeTime{
	my ($class,$time) = self_or_default(@_);
	my @t = ($time =~ /\s(\d+)\:(\d+)\:\d+$/);
	$time = $t[0].":".$t[1];
	return $time;
}

sub MakeDateTime{
	my ($class,$time) = self_or_default(@_);
	if ($time eq "") { return undef; }
	my @t = ($time=~/(\d+)-(\d+)-(\d+)\s(\d+):(\d+):(\d+)/);
	$t[2] += 0;
	$time = $t[2]." ".$month[$t[1]-1]." ".$t[0].", $t[3]:$t[4]";
	return $time;
}

sub isInArray{
    my ($class,$element,$array) = self_or_default(@_);
    my $match = 0;
        
    if (ref($array) && defined($element)){
        foreach (@{$array}){
	    if ($element eq $_){
		$match = 1;
		last;
	    }
	}
	return $match;
    } else {
	return undef;
    }
}

sub localeGrp {
    my ($class,$grp_id) = self_or_default(@_);
    
    if ($grp_id < 1 || $grp_id > ($#totalGroups+1)){
        return undef;
    }

    return $totalGroups[$grp_id-1];
}

sub messages {
    my ($class,$message) = self_or_default(@_);
    if (exists $messages{$message}) {
	    return $messages{$message};
    } else {
    	return "undefined error";
    }
}

sub transPath {
	my $path = shift;
	my @par = ();
	my @return = ();

	if (! defined($$path) || length($$path) < 1 || length($$path) > 150){
		return @return;
	}

	foreach ( split(/\//, $$path) ) {
		if (defined($_) && $_ =~ /^[\w\.\-\+]+$/ ) {
			push @par,$_;
		}
	}

	my $i = 0;
	foreach my $control (@_) {
		unless(defined($par[$i])) {
			last;
		}
		if (ref($control) eq "ARRAY"){
			if (isInArray($par[$i],$control)){
				push @return,$par[$i];
			} else {
				push @return, undef;
			}
		} elsif (! ref($control)) {
			# "w" - all good symbols
			if ($control eq "w"){
				push @return,$par[$i];
			# "s" - "w" without "+" & "."
			} elsif ($control eq "s"){
				if ($par[$i] =~ /^[\w\-]+$/){
					push @return,$par[$i];
				} else {
					push @return, undef;
				}
			# "d" - only digits
			} elsif ($control eq "d"){
				if ($par[$i] =~ /\D/) {
					push @return, undef;
				} else {
					push @return,$par[$i];
				}
			}
		} else {
			push @return, undef;
		}
		$i++;
	}

	return @return;
}

1;
