#!/usr/bin/perl
#
# script:       forum.pl
# description:  forum engine
# author:       Vladimir Lettiev aka crux <crux@altlinux.ru> (c) 2004
# version:      0.1

use strict;
use CGI qw/ param cookie /;
use DBI;
use FindBin qw($Bin);
use lib "$Bin/adm";
use altbase;

#> My constants
my ($select_Q,@row,@ary,$row,$sth,$sth2,$dbh);
my @sections = ("add","del");
my ($nick,$uid) = (undef,undef);
my $text = "";
my $subtext = "";
my $messPerPage = 10;
my $startMess = 0;

my $cgi = new CGI;
my $alt = new altbase;

#> Load html fragments
if (! $alt->init(module => "inform", script => "forum.pl") ){
    print $altbase::header . $altbase::errStr;
    exit;
}

my ($srpm,$page,$id,$action,$pid,$sid) = myTransPath();

unless (defined($srpm)){
    print $altbase::header . "error [forum.pl:32]. no forum specified";
    exit;
}

#> Connection to MySQL
$dbh = DBI->connect($altbase::dsn,$altbase::dbUser,$altbase::dbPass);

my $session = cookie('ses');

if (defined($session) && length($session) == 15 && $session =~ /^\w+$/) {
    my $selectName_Q = "SELECT nick,uid FROM users WHERE session=? AND (UNIX_TIMESTAMP(NOW()) - UNIX_TIMESTAMP(time)) < ?";
    $sth = $dbh->prepare($selectName_Q);
    $sth->execute($session,$altbase::TTL);
    ($nick,$uid) = $sth->fetchrow_array;
    if ($nick) {
	my $updateTime_Q = "UPDATE users SET time=NOW() WHERE session=?";
	$sth = $dbh->prepare($updateTime_Q);
	$sth->execute($session);
    }
    $sth->finish();
}

# Show discussion
if (defined($id)){
    my $getTopicMess = "SELECT t.time, t.header, t.message, t.uid, u.nick FROM topic as t, users as u WHERE t.id=? AND t.uid=u.uid AND t.srpm=?";
    $sth = $dbh->prepare($getTopicMess);
    $sth->execute($id,$srpm);
    @row = $sth->fetchrow_array();
    if (@row) {
	$text = $alt->parser(
		    HTML => "topicMess",
		    vars => {
			"nick" => tagClean($row[4]),
			"time" => MakeDateTime($row[0]),
			"header" => tagClean($row[1]),
			"mess" => tagClean($row[2]),
			"path" => $altbase::path,
			"srpm" => $srpm,
			"id" => $id
		    },
		    iflist => {
			"auth" => (defined($nick) && ! defined($action))?1:0
		    }
	);
	
	my @Data = ();
	
	my $getAnsMess_Q = "SELECT d.time, d.reply, d.message, d.uid, u.nick, d.id, d.pid \
	FROM discussion as d, users as u \
	WHERE d.uid=u.uid AND d.srpm=? AND d.tid=? ORDER BY d.pid ASC, d.time ASC";

        $sth = $dbh->prepare($getAnsMess_Q);
	$sth->execute($srpm,$id);
        while (@ary = $sth->fetchrow_array()){

	    my ($who,$when,$reply,$text);

	    if ($ary[6] != 0) {
		my $getBack_Q = "SELECT u.nick, d.time, LENGTH(d.reply) FROM discussion as d, users as u WHERE d.id=? AND d.uid=u.uid";
		$sth2 = $dbh->prepare($getBack_Q);
		$sth2->execute($ary[6]);
		($who,$when,$reply) = $sth2->fetchrow_array;
		$sth2->finish();
	    } else {
		$who = $row[4];
		$when = $row[0];
		$reply = 1;
	    }

	    $text = $alt->parser(
		    HTML => "answerMess",
		    vars =>{
			"nick" => tagClean($ary[4]),
			"time" => MakeDateTime($ary[0]),
			"header" => tagClean($ary[1]),
			"mess" => tagClean($ary[2]),
			"path" => $altbase::path,
			"srpm" => $srpm,
			"id" => $id,
			"pid" => $ary[5],
			"who" => tagClean($who),
			"when" => MakeDateTime($when),
			"offset" => "%W%"
		    },
		    iflist=>{
			"auth" => (defined($nick) && ! defined($action))?1:0,
			"reply" => $reply
		    }
	    );
	    
	    push @Data,[$ary[5], $ary[6], $text, "0"];
	}

	$text .= &makeTree(\@Data);
	$row = $#Data+1;
	undef @Data;

	my $updReads_Q = "UPDATE topic SET reads=reads+1 WHERE id=?";
	$sth = $dbh->prepare($updReads_Q);
	$sth->execute($id);
	$sth->finish();
	
	if (defined($nick)){
		my $updCountAns_Q = "REPLACE fcookie VALUES(?,?,?,?)";
		$sth = $dbh->prepare($updCountAns_Q);
		$sth->execute($uid,$srpm,$id,$row);
		$sth->finish();
	}
	
    } else {
	$text = $alt->messages("noSuchTopic");
    }

#      
} elsif ($action eq "addmess" && defined($nick)) {
    $text = $alt->parser(
		HTML => "newmess",
		vars => {
		    "path" => $altbase::path,
		    "srpm" => tagClean($srpm)
		},
		iflist => {}
    );

#      
} elsif ($action eq "answer" && defined($nick)) {

    my $getMess_Q;
    
    if (defined($sid)){
	$getMess_Q = "SELECT d.time, d.reply, d.message, u.nick FROM discussion as d, users as u \
	WHERE d.id=? AND d.uid=u.uid AND d.srpm=? ";
	$sth = $dbh->prepare($getMess_Q);
	$sth->execute($sid, $srpm);
    } else {
        $getMess_Q = "SELECT t.time, t.header, t.message, u.nick FROM topic as t, users as u \
        WHERE t.id=? AND t.uid=u.uid AND t.srpm=?";
	$sth = $dbh->prepare($getMess_Q);
	$sth->execute($pid, $srpm);
    }

    @row = $sth->fetchrow_array();
    if (@row) {
	my $reHeader = tagClean($row[1]);
	if ($reHeader !~ /^Re/) {
	    $reHeader = "Re: ".$reHeader;
	} else {
    	    $reHeader =~ s/^Re\[(\d+)\]:\s/"Re[". ($1+1) . "]: "/e;
	    $reHeader =~ s/^Re:\s/Re[2]: /;
	}

        $text = $alt->parser(
		HTML => "answerForm",
		vars => {
		    "path" => $altbase::path,
		    "srpm" => tagClean($srpm),
		    "nick" => tagClean($row[3]),
		    "time" => MakeDateTime($row[0]),
		    "header" => tagClean($row[1]),
		    "mess" => tagClean($row[2]),
		    "srpm" => $srpm,
		    "pid" => $pid,
		    "sid" => $sid,
		    "reHeader" => $reHeader,
		},
		iflist => {
		    "sid" => defined($sid)?1:0
		}
	);
    
    } else {
	$text = $alt->messages("noSuchTopic");
    }



# Show list of topics
} else {
    $startMess = ($page-1)*$messPerPage;
    my $bold = 1;

    my $selTopics_Q = "SELECT t.id, t.time, t.header, t.reads, t.answs, u.nick, t.lasttime, LEFT(t.message,50) from topic as t, users as u WHERE t.srpm=? and t.uid=u.uid ORDER BY t.lasttime DESC LIMIT $startMess,$messPerPage";
    $sth = $dbh->prepare($selTopics_Q);
    $sth->execute($srpm);
    while (@row = $sth->fetchrow_array){

	if (defined($nick)){
	    my $selUserAnsCount_Q = "SELECT ans FROM fcookie WHERE id=? AND srpm=? AND uid=?";
	    $sth2 = $dbh->prepare($selUserAnsCount_Q);
	    $sth2->execute($row[0],$srpm,$uid);
	    $row = $sth2->fetchrow_array();
	    if (defined($row) && $row[4] == $row){
		$bold = 0;
	    } else {
		$bold = 1;
	    }
	}

	if ($row[4] != 0){
	    my $selUserAns_Q = "SELECT u.nick, d.time FROM discussion as d, users as u WHERE d.uid=u.uid \
	    AND d.tid=? AND d.srpm=? ORDER BY d.time DESC LIMIT 1";
	    $sth2 = $dbh->prepare($selUserAns_Q);
	    $sth2->execute($row[0],$srpm);
	    ($row[8],$row[6]) = $sth2->fetchrow_array;
	    $sth2->finish();
	}
		
        $text .= $alt->parser(
                        HTML => "topic",
                        vars => {
                            "path" => $altbase::path,
                            "srpm" => $srpm,
                            "id" => $row[0],
                            "author" => tagClean($row[5]),
                            "time" => MakeDateTime($row[1]),
                            "header" => tagClean($row[2]),
                            "messpart" => tagHardClean($row[7]),
                            "lasttime" => MakeDateTime($row[6]),
			    "lastnick" => tagClean($row[8]),
                            "reads" => $row[3],
                            "answs" => $row[4]
                        },
                        iflist => {
                	    "bold" => $bold,
			    "last" => ($row[4] !=0)?1:0
                        }
        );
    }
    
    my $selCount_Q = "SELECT count(*) from topic WHERE srpm=?";
    $sth = $dbh->prepare($selCount_Q);
    $sth->execute($srpm);
    $row = $sth->fetchrow_array();
    if ($row > $messPerPage) {
        my $pg = sprintf("%u", ($row/$messPerPage));
        if ($row%$messPerPage != 0 ) {
           $pg++; 
        }
        for (my $i=1; $i<=$pg; $i++){
            $subtext .= $alt->parser(
                            HTML => "link2page",
                            vars => {
                                "path"=>$altbase::path,
                                "page"=>$i,
                                "srpm"=>$srpm,
                            },
                            iflist => {
                                "isCur" => ($i==$page)?1:0
                            }
            );
        }
    }
    
    $text = $alt->parser( HTML=>"topicList",
                          vars=>{"topics"=>$text, "pages"=>$subtext},
                          iflist => {"pages"=>$subtext?1:0});
}

$text = $alt->parser (
            HTML => "main",
            vars => { "content" => $text, "name" => $srpm, "path" => $altbase::path, },
            iflist => { "auth" => (defined($nick) && ! defined($action))?1:0 }
);


print $altbase::header;
print $text;

sub myTransPath {
    my $path = $ENV{"PATH_INFO"};
    my (@path,@par);
    my @return = (undef,1,undef,undef,undef,undef);

    if (! defined($path) || length($path) < 1 || length($path) > 150){
        return @return;
    }

    @path = split(/\//, $path);
    foreach(@path){
        if (defined($_) && $_ =~ /^[\w\.\-\+]+$/ ) {
            push(@par,$_)
        }
    }
    if (! @par){
        return @return;
    }

    if ($par[0] =~ /^([\w\-\.\+]+)$/){
        $return[0] = $1;
        if (defined($par[1])){
            if ($par[1] =~ /^(\d+)\.html$/ && $1 < $altbase::maxId ){
                $return[2] = $1;
            } elsif ($par[1] =~ /^page(\d+)$/ && $1 < $altbase::maxPage && $1 > 0){
                $return[1] = $1;
            } elsif ($par[1] eq "addmess.html") {
		$return[3] = "addmess";
	    } elsif ($par[1] =~ /^ans(\d+)$/ && $1 < $altbase::maxId) {
		$return[3] = "answer";
		$return[4] = $1;
		if (defined($par[2]) && $par[2] =~ /^mess(\d+)\.html$/ && $1 < $altbase::maxId ){
		    $return[5] =$1;
		}
	    }
        }
    }

    return @return;
}

sub makeTree {
    my $array = shift;
    my $oldpid = -1;
    my $i = 0;
    my (%A,%B,@All,$text,$pl,$level);
    my $offset = 15;

    foreach my $row (@$array){
	if ($oldpid == $$row[1]){
            $B{$$row[1]}++;
	} else {
	    $A{$$row[1]} = $i;
	    $oldpid = $$row[1];
	}
        $i++;
    }

    foreach (sort {$a<=>$b} keys %A){
	if (! @All){
            $pl = 0;
            $level = 0;
        } else {
            for my $i (0 .. $#All){
                if ($All[$i][0] == $_) {
                    $level = $All[$i][3] + 1;
                    $pl = $i+1;
	            last;
                }
            }
        }
        for my $i ($A{$_}..$A{$_}+$B{$_}){
            $$array[$i][3] = $level;
        }
        splice(@All,$pl,0,@{$array}[$A{$_}..$A{$_}+$B{$_}]);
    }

    foreach my $row (@All){
	$$row[2] =~ s/%W%/($$row[3]+1)*$offset/e;
	$text .= $$row[2];
    }

    return $text;

}