#!/usr/bin/perl
#
# script:       my.pl
# description:  personal engine
# author:       Vladimir Lettiev aka crux <crux@altlinux.ru> (c) 2004
# version:      0.1

use strict;
use CGI qw/param cookie/;
use DBI;
use FindBin qw($Bin);
use lib "$Bin/adm";
use altbase;
use Time::HiRes qw/ gettimeofday tv_interval /;
my $nowTime = [gettimeofday];

#> My constants
my ($select_Q,@row,$row,$sth,$dbh,$section);
my @sections = ("pref","logout");
my $text = "";
my $nick = undef;
my $uid = undef;

my $cgi = new CGI;
my $alt = new altbase;

#> Load html fragments
if (! $alt->init(module => "inform", script => "my.pl") ){
    print $altbase::header . $altbase::errStr;
    exit;
}

#> Connection to MySQL
$dbh = DBI->connect($altbase::dsn,$altbase::dbUser,$altbase::dbPass);

my $session = cookie('ses');
if (defined($session) && length($session) == 15 && $session =~ /^\w+$/) {
    my $selectName_Q = "SELECT nick,uid FROM users WHERE session=? AND (UNIX_TIMESTAMP(NOW()) - UNIX_TIMESTAMP(time)) < ?";
    $sth = $dbh->prepare($selectName_Q);
    $sth->execute($session,$altbase::TTL);
    ($nick,$uid) = $sth-> fetchrow_array;
    if ($nick) {
        my $updateTime_Q = "UPDATE users SET time=NOW() WHERE session=?";
        $sth = $dbh->prepare($updateTime_Q);
        $sth->execute($session);
    }
    $sth->finish();
}


if ($ENV{"REQUEST_METHOD"} eq "POST" && defined($nick)) {
    my ($temp,$newtype);
    
    my $selPrefs_Q = "SELECT type FROM userprefs WHERE uid=? AND rpm=?";
    my $dropPrefs_Q = "DELETE FROM userprefs WHERE uid=? AND rpm=?";
    my $updatePrefs_Q = "UPDATE userprefs SET type=? WHERE uid=? AND rpm=?";
    
    if (defined( $cgi->param('book') )){
        $newtype = "pref";
    } elsif (defined( $cgi->param('pref') )){
        $newtype = "bookmark";    
    } else {
        print $altbase::redirect . $altbase::path . "/my\n\n";
        exit;
    }
    
    foreach my $rpm (grep {/^[\w\-\+\.]+$/} $cgi->param){
        if ($cgi->param($rpm) eq "on" && length($rpm) <= 100) {
            $sth = $dbh->prepare($selPrefs_Q);
            $sth->execute($uid,$rpm);
            $temp = $sth->fetchrow_array();
            if ($temp eq "all"){
                $sth = $dbh->prepare($updatePrefs_Q);
                $sth->execute($newtype,$uid,$rpm);
            } elsif ($temp) {
                $sth = $dbh->prepare($dropPrefs_Q);
                $sth->execute($uid,$rpm);
            }
        }
    }
    
    print $altbase::redirect . $altbase::path . "/my\n\n";
    exit;
}

my $type = $cgi->param('type');

if ($type eq "topline"){
    
    my $ip=$ENV{'REMOTE_ADDR'};
 
    my $delOldIp_Q = "DELETE FROM counter WHERE time < DATE_SUB(NOW(), INTERVAL 30 MINUTE)";
    $sth = $dbh->prepare($delOldIp_Q);
    $sth->execute();
    my $findIp_Q="SELECT count(*) FROM counter WHERE ip=?";
    $sth = $dbh->prepare($findIp_Q);
    $sth->execute($ip);
    $row = $sth->fetchrow_array;
    if ($row == 0){
        my $updVisits_Q="UPDATE visit SET val=val+1";
        $sth = $dbh->do($updVisits_Q);
    }
    my $update_Q="REPLACE counter SET time=NOW(),ip=?";
    $sth = $dbh->prepare($update_Q);
    $sth->execute($ip);

    my $selVal_Q="SELECT val FROM visit";
    $sth = $dbh->prepare($selVal_Q);
    $sth->execute();
    my $val = $sth->fetchrow_array;

    my $selLastUpdate_Q = "SELECT MAX(buildtime) FROM srpm";
    ($row) = $dbh->selectrow_array($selLastUpdate_Q);
    
    my $selSrpmCount_Q = "SELECT count(name) FROM srpm WHERE age>0";
    my ($srpmCount) = $dbh->selectrow_array($selSrpmCount_Q);

    $text = $alt->parser(
            HTML => "topline",
            vars => {
                    "date" => &MakeDate($row),
                    "count" => $val,
		    "srpms" => $srpmCount,
                    "user" => tagClean($nick),
                    "path" => $altbase::path
            },
            iflist => {
                "user" => $nick?1:0
            }
    );

} elsif ($type eq "pref") {

    if (! defined($nick)){

        $text = "error: [my.pl:73] user is not defined!";

    } else {

        my $srpm  = $cgi->param('srpm');
        my $check = $cgi->param('check');
        my $book  = $cgi->param('book');

        if ($check eq "on" && $book eq "on"){
            $type = "all";
        } elsif ($check eq "on") {
            $type = "pref";
        } elsif ($book eq "on") {
            $type = "bookmark";
        } else {
            $type = "drop";
        }
        
        if ($srpm =~ /^[\w\+\.\-]+$/ && length($srpm) <= 100) {
        
            my $selSrpm_Q = "SELECT CONCAT(srpm.version,'-',srpm.release), count(bugs.bug_id) FROM srpm, \
	    rpm LEFT JOIN bugs ON bugs.rpm=rpm.name WHERE srpm.name=? AND rpm.srpm=srpm.name GROUP BY rpm.srpm";
            
            $sth = $dbh->prepare($selSrpm_Q);
            $sth->execute($srpm);
            @row = $sth->fetchrow_array();
            if (@row) {
                if ($type eq "drop"){
                    my $deletePref_Q = "DELETE from userprefs WHERE uid=? AND rpm=?";
                    $sth = $dbh->prepare($deletePref_Q);
                    $sth->execute($uid,$srpm);
                } else {
                    my $changePref_Q = "REPLACE userprefs VALUES(?,?,?,?,?)";
                    $sth = $dbh->prepare($changePref_Q);
                    $sth->execute($uid,$srpm,@row,$type);
                }

                $text = $alt->parser(
                            HTML => "result",
                            vars => {
                                "srpm" => tagClean($srpm)
                            },
                            iflist =>{
                                "check" => ($type eq "all" || $type eq "pref")?1:0,
                                "book" => ($type eq "all" || $type eq "bookmark")?1:0
                            }
                );
            } else {
                $text = "error: [my.pl:100] no such srpm";
            }
        } else {
            $text = "error: [my.pl:103] dont play with parameters, i dont like it...";
        }
    }

    $text = $alt->parser(
                HTML => "prefsChange",
                vars => {
                    "content" => $text
                },
                iflist => {}
    );

} elsif (defined($nick)) {
    ($section) = transPath(\$ENV{"PATH_INFO"},\@sections);

    if (! defined($section)){
        my $selSrpm_Q = "SELECT s.name, s.summary, CONCAT(s.version,'-',s.release), \
        u.version, u.bugscount, u.type FROM userprefs as u, srpm as s \
        WHERE u.uid=? AND u.rpm=s.name ORDER BY 1 ASC";
        
        my $rpmlist  = "";
        my $booklist = "";
	my ($sth2,$allmess,$readmess);
        
        $sth=$dbh->prepare($selSrpm_Q);
        $sth->execute($uid);
        while(@row = $sth->fetchrow_array){
            if ($row[5] eq "pref" || $row[5] eq "all") {

		my $selAnsCount_Q = "SELECT SUM(answs) + count(*) FROM topic WHERE srpm=?";
		$sth2 = $dbh->prepare($selAnsCount_Q);
		$sth2->execute($row[0]);
		$allmess = $sth2->fetchrow_array;

		$selAnsCount_Q = "SELECT SUM(ans) + count(*) FROM fcookie WHERE srpm=? AND uid=?";
		$sth2 = $dbh->prepare($selAnsCount_Q);
		$sth2->execute($row[0],$uid);
		$readmess = $sth2->fetchrow_array;

		my $selBugsCount_Q = "SELECT count(b.bug_id) FROM rpm as r LEFT JOIN bugs as b ON r.name=b.rpm \
		WHERE r.srpm=?";
	        $sth2 = $dbh->prepare($selBugsCount_Q);
		$sth2->execute($row[0]);
		$row[6] = $sth2->fetchrow_array;
				   
                $rpmlist .= $alt->parser(
                            HTML => "rpmrow",
                            vars =>{
                                "path" => $altbase::path,
                                "srpm" => $row[0],
                                "summary" => tagClean($row[1]),
                                "newversion" => tagClean($row[2]),
                                "newmess" => ($allmess-$readmess),
                                "newbugs" => ($row[6]-$row[4])
                            },
                            iflist => {
                                "newversion" => ($row[2] eq $row[3])?0:1,
                                "newmess" => ($allmess-$readmess),
                                "newbugs" => ($row[4] eq $row[6])?0:1
                            }
                );
            }
            if ($row[5] eq "bookmark" || $row[5] eq "all") {
                $booklist .= $alt->parser(
                            HTML => "bookrow",
                            vars =>{
                                "path" => $altbase::path,
                                "srpm" => $row[0],
                                "summary" => tagClean($row[1])
                            },
                            iflist => {}
                );
            }
        }
        
        $text= $alt->parser (
                HTML => "main",
                vars => {
                    "nick" => tagClean($nick),
                    "rpmlist" => $rpmlist,
                    "bookmarks" => $booklist,
		    "timeElapsed" => tv_interval ($nowTime)	
                },
                iflist => {
                    "rpmlist" => ($rpmlist)?1:0,
                    "bookmarks" => ($booklist)?1:0
                }
         );
    } elsif ($section eq "logout") {
        my $logout_Q = "UPDATE users SET time='' WHERE session=?";
        $sth = $dbh->prepare($logout_Q);
        $sth->execute($session);
        $text = $alt->parser(
                    HTML=>"logout",
                    vars=>{ "nick" => tagClean($nick)},
                    iflist=>{}
        );
    } elsif ($section eq "pref") {
    
    }
    
} else {
    $text = "error [my.pl:87]: user not defined, sorry";
}

print $altbase::header;
print $text;
