#!/usr/bin/perl
#
# script:       packager.pl
# description:  show information about maintainer
# author:       Vladimir Lettiev aka crux <crux@altlinux.ru> (c) 2004-2005
# version:      0.02

use strict;
use CGI qw/param/;
use DBI;
use FindBin qw($Bin);
use lib "$Bin/adm";
use altbase;

#> My constants
my ($select_Q,@ary,@row,$row,$sth,$dbh);
my $text = "";
my @sections = ("main","srpms","bugs");

my $cgi = new CGI;
my $alt = new altbase;

#> Load html fragments
if (! $alt->init(module => "inform", script => "packager.pl") ){
    print $altbase::header . $altbase::errStr;
    exit;
}

my ($packager,$section) = transPath(\$ENV{"PATH_INFO"},"s",\@sections);
my $show = $cgi->param('show');

my $selMainInfo_Q = "SELECT packager, type FROM maintainers WHERE mail=?";

if (! defined($packager)){
    print $altbase::header;
    print (($show eq "title")?"unknown":"error [packager.pl:38]");
    exit;
} else {
    #> Connection to MySQL
    $dbh = DBI->connect($altbase::dsn,$altbase::dbUser,$altbase::dbPass);
    
    $sth = $dbh->prepare($selMainInfo_Q);
    $sth->execute($packager);
    @row = $sth->fetchrow_array();
    $sth->finish();
    if (! @row){
        print $altbase::header;
        print (($show eq "title")?"unknown":"error [packager.pl:48]");
        exit;
    } elsif ($show eq "title"){
        print $altbase::header;
        print $packager;
        exit;
    }

    $row[0] =~ s/\s*<.+$//;
    my ($rpmlist,$count) =  &rpmList($packager);

    if ($row[1] eq "team") {
        $text = $alt->parser(
                HTML => "team",
                vars => {
                    "members" => &showList($packager),
                    "packager" => $row[0],
                    "info" => "",
                    "rpmlist" => $rpmlist,
                    "count" => $count
                },
                iflist => {}
        );
    } else {
        
        if (! defined($section)) {
            
            $text = $alt->parser(
                    HTML => "packager",
                    vars => {
                        "packager" => $row[0],
                        "info" => "",
                        "count" => $count
                    },
                    iflist => {}
            );
        
        } elsif ($section eq "srpms"){
            
            $text = $alt->parser(
                    HTML => "rpmlist",
                    vars => {
                        "rpmlist" => $rpmlist,
                        "count" => $count
                    },
                    iflist => {}
            );

        } elsif ($section eq "bugs"){

            my $getBugsList_Q = "SELECT srpm.name, count(bugs.bug_id), srpm.summary FROM bugs, rpm, srpm, maintainers WHERE bugs.assigned=? AND rpm.name=bugs.rpm AND rpm.srpm=srpm.name AND maintainers.mail=srpm.packager AND ((srpm.packager!=bugs.assigned AND maintainers.type='team') OR (srpm.packager=bugs.assigned)) AND srpm.age > 0 GROUP BY srpm.name ORDER BY 2 DESC, srpm.name ASC";
            $sth = $dbh->prepare($getBugsList_Q);
            $sth->execute($packager);
            my $i = 0;
            my $j = 0;
            while(@ary = $sth->fetchrow_array){
                   $i+=$ary[1];
                   $j++;
                   $text .= $alt->parser(
                            HTML => "bug",
                            vars => {
                                "path" => $altbase::path,
                                "rpm" => $ary[0],
                                "count" => $ary[1],
                                "summary" => tagClean($ary[2])
                            },
                            iflist => {}
                    );
            }
            $text = $alt->parser(
                    HTML => "bugslist",
                    vars => {
                        "content" => $text,
                        "count" => $i,
                        "rpmcnt" => $j
                    },
                    iflist => {
                        "bugs" => ($i==0)?0:1
                    }
            );

        } else {
        
        }
        $text = $alt->parser(
                HTML => "titleman",
                vars => {
                    "packager" => $row[0],
                    "mail" => $packager,
                    "path"=> $altbase::path,
                    "content"=> $text
                },
                iflist => {
                    "main" => (! defined($section))?1:0,
                    "packages" => ($section eq "srpms")?1:0,
                    "bugs" => ($section eq "bugs")?1:0,
                }
        );
    }

    $dbh->disconnect();
    print $altbase::header;
    print $text;
}

sub rpmList {
    my $mail = shift;
    my ($sth,@row,$text,$count,$url);
    my $selMainRPM_Q = "SELECT name,CONCAT(version,'-',release),UNIX_TIMESTAMP(NOW()) - UNIX_TIMESTAMP(buildtime), buildtime, url, summary FROM srpm WHERE packager=? AND age>0 ORDER BY 1 ASC";

    $sth = $dbh->prepare($selMainRPM_Q);
    $sth->execute($mail);
    while(@row = $sth->fetchrow_array()){
        $count++;
#        if ($count%2 == 1) {
#            $text .= "<tr>";
#        }
        tagConv(\$row[4],20);
        $text .= $alt->parser(
                HTML => "srpm",
                vars => {
                    "path" => $altbase::path,
                    "name" => $row[0],
                    "release" => $row[1],
                    "date" => MakeDateTime($row[3]),
                    "old" => sprintf("%.1f",$row[2]/(3600*24*7)),
                    "url" => ($row[4])?$row[4]:"-",
                    "summary" => tagClean($row[5])
                },
                iflist => {}
        );
    }

    return $text,$count;
}

sub showList {
    my $mail = shift;
    my ($text,$sth,$row,@members,$imail,@row);

    my $selManInfo_Q = "SELECT DISTINCT m.packager, m.type, m.mail FROM maintainers as m, changelogs as c, srpm as s WHERE s.packager=? AND s.name=c.rpm AND m.mail=c.packager";
    $sth = $dbh->prepare($selManInfo_Q);
    $sth->execute($mail);
       
    while (@row = $sth->fetchrow_array()){
            $text .= $alt->parser(
                HTML => "member",
                vars => {
                "path" => $altbase::path,
                "mail" => $row[2],
                "packager" => $row[0],
            },
            iflist => {
                "team" => ($row[1] eq "team")?0:1
            }
            );
    }
    $sth->finish();
    
    return $text;
}
