#!/usr/bin/perl
#
# script:       srpm.pl
# description:  show full information about srpm packet
# author:       Vladimir Lettiev aka crux <crux@altlinux.ru> (c) 2004
# version:      0.4

use strict;
use CGI qw/param cookie/;
use DBI;
use FindBin qw($Bin);
use lib "$Bin/adm";
use altbase;

#> My constants
my ($select_Q,@row,@ary,$row,$sth,$dbh);
my @sections = ("docs","changelog","bugs","allbugs","spec","get");
my $text = "";

my $cgi = new CGI;
my $alt = new altbase;

#> Load html fragments
if (! $alt->init(module => "inform", script => "srpm.pl") ){
    print $altbase::header . $altbase::errStr;
    exit;
}

my ($srpm,$section) = transPath(\$ENV{"PATH_INFO"},"w",\@sections);

my $show = param('show');
    
if (! defined($srpm)){
    print $altbase::header;
    print (($show eq "title")?"unknown":"error [stats.pl:38]");
    exit;
} else {
    #> Connection to MySQL
    $dbh = DBI->connect($altbase::dsn,$altbase::dbUser,$altbase::dbPass);

    my $findSrpm_Q = "SELECT count(name) FROM srpm WHERE name=? AND age>0";

    $sth = $dbh->prepare($findSrpm_Q);
    $sth->execute($srpm);
    $row = $sth->fetchrow_array();
    $sth->finish();
    if ($row == 0){
        print $altbase::header;
        print (($show eq "title")?"unknown":"error [stats.pl:52]");
        exit;
    } elsif ($show eq "title"){
        print $altbase::header;
        print $srpm;
        exit;
    }

    my $selSrpmInfo_Q = "SELECT s.version, s.release, g.rpmgroup, m.packager, s.buildtime, \
	s.summary, s.description, s.packager, m.type, s.copyright, \
	UNIX_TIMESTAMP(NOW())-UNIX_TIMESTAMP(s.buildtime), \
        s.size, s.url, g.id FROM srpm as s, maintainers as m, groups as g \
        WHERE m.mail=s.packager AND g.id=s.rpmgroup AND s.name=?";

    $sth = $dbh->prepare($selSrpmInfo_Q);
    $sth->execute($srpm);
    if (@row = $sth->fetchrow_array()){
        tagConv(\$row[6]);
        tagConv(\$row[12]);

        if (defined($section)){

            if ($section eq "spec"){

                my $SelSpec_Q = "SELECT spec FROM spec WHERE rpm=?";
                $sth = $dbh->prepare($SelSpec_Q);
                $sth->execute($srpm);
                $row = $sth->fetchrow_array();
                specColorer(\$row);
                $text = $alt->parser(
                            HTML => "spec",
                            vars => {
                                "text" => $row,
                                "name" => $srpm,
                                "path" => $altbase::path
                            },
                            iflist => {}
                            );

            } elsif ($section eq "changelog"){

                my $SelChangelog_Q = "SELECT name, time, content FROM changelogs WHERE rpm=? \
                ORDER BY time DESC";
                $sth = $dbh->prepare($SelChangelog_Q);
                $sth->execute($srpm);
                while(@ary = $sth->fetchrow_array()){
                    $ary[0] =~ s/\@/ at /;
                    $text .= $alt->parser(
                            HTML => "oneChange",
                            vars => {
                                "time" => MakeDate($ary[1]),
                                "name" => tagClean($ary[0]),
                                "text" => changeFormat($ary[2])
                            },
                            iflist => {}
                            );
                }
            } elsif ($section eq "get"){

                my $SelMirrors_Q = "SELECT mirror FROM mirrors ORDER BY 1 ASC";
                $sth = $dbh->prepare($SelMirrors_Q);
                $sth->execute();
                while (@ary = $sth->fetchrow_array()){
                    $text .= $alt->parser(
                            HTML => "mirror",
                            vars => {
                                "mirror" => $ary[0],
                                "name" => $srpm,
                                "version" => $row[0],
                                "release" => $row[1]
                            },
                            iflist => {}
                            );

                }
                $text = $alt->parser(
                            HTML => "getSource",
                            vars => {
                                "list" => $text
                            },
                            iflist => {}
                            );

            } elsif ($section eq "docs"){

            # Not finished yet

            } elsif ($section eq "bugs" || $section eq "allbugs"){

                my $i = 0;
		my %countBugs = ();
		my $statusFilter = "";
	        my @statusFilter = ();

		foreach ('UNCONFIRMED','NEW','ASSIGNED','REOPENED') {
		    push @statusFilter,"bugs.status='$_'";
		}
		$statusFilter = " AND (" . join(' OR ',@statusFilter) . ")";

                my $selBugs_Q = "SELECT bugs.bug_id, bugs.severety, bugs.priority, bugs.reporter, \
		bugs.status, bugs.resolution, bugs.description from bugs LEFT JOIN rpm ON rpm.name=bugs.rpm\
		WHERE rpm.srpm=?";
		$selBugs_Q .= $statusFilter if ($section eq "bugs");
		$selBugs_Q .= " ORDER BY resolution ASC, status ASC, severety DESC, priority ASC, bug_id DESC";
                
		$sth = $dbh->prepare($selBugs_Q);
                $sth->execute($srpm);
                while (@ary = $sth->fetchrow_array){
                    $i++;
                    $ary[3] =~ s/\@/ at /;
                    $text .= $alt->parser(
                            HTML => "bug",
                            vars => {
                                "id" => $ary[0],
                                "severety" => $ary[1],
                                "priority" => $ary[2],
                                "reporter" => tagClean($ary[3]),
                                "status" => $ary[4],
                                "resolution" => ($ary[5])?$ary[5]:"&nbsp;",
                                "description" => tagClean($ary[6])
                            },
                            iflist => {}
                            );
                
                }
		
		my $selBugsCount_Q = "SELECT count(bugs.bug_id) FROM bugs LEFT JOIN rpm ON " .
		"rpm.name=bugs.rpm WHERE rpm.srpm=?";
		$selBugsCount_Q .= $statusFilter if ($section eq "allbugs");
		
		$sth = $dbh->prepare($selBugsCount_Q);
                $sth->execute($srpm);
		$countBugs{"get"} = $sth->fetchrow_array();
		$sth->finish();
		
		if ($section eq "bugs") {
		    $countBugs{"allBugsCount"}  = $countBugs{"get"};
		    $countBugs{"openBugsCount"} = $i;
		} else {
		    $countBugs{"allBugsCount"}  = $i;
		    $countBugs{"openBugsCount"} = $countBugs{"get"};
		}
		
                $text = $alt->parser(
                            HTML => "bugslist",
                            vars => {
                                "content" => $text,
				"path" => $altbase::path,
				"name" => $srpm,
                                "allBugsCount" => $countBugs{"allBugsCount"},
				"openBugsCount" => $countBugs{"openBugsCount"},
				"closedBugsCount" => $countBugs{"allBugsCount"} - $countBugs{"openBugsCount"}
                            },
                            iflist => {
                                "bugs" => ($countBugs{"allBugsCount"} == 0)?0:1,
				"open" => ($section eq "bugs" && $countBugs{"allBugsCount"} >0 )?1:0,
				"all" => ($section eq "allbugs" && $countBugs{"allBugsCount"} >0 )?1:0
                            }
                            );
		

            } else {
                print $altbase::header;
                print "internal error [srpm.pl:94]";
                exit;
            }

        } else {

            $text = $alt->parser(
                    HTML => "package",
                    vars => {
                        "path" => $altbase::path,
                        "name" => $srpm,
                        "version" => $row[0],
                        "release" => $row[1],
                        "packager" => $row[3],
                        "date" => MakeDateTime($row[4]),
                        "url" => $row[12],
                        "sub" => sprintf("%.1f",$row[10]/(3600*24*7)),
                        "summary" => $row[5],
                        "description" => $row[6],
                        "mail" => $row[7],
                        "size" => &sizeOf($row[11]),
                        "license" => $row[9],
                        "mainlist" => &allMan($srpm),
                        "rpmlist" => &allRpm($srpm)
                    },
                    iflist => {
                        "team" => ($row[8] eq "team")?0:1,
                        "url" => ($row[12] eq "")?0:1
                    }
                );
        }

        $text = $alt->parser(
                    HTML => "title",
                    vars => {
                        "content" => $text,
                        "navigator" => &getNavigator($row[7],$row[13],$srpm),
                        "group" => &localeGrp($row[13]),
                        "name" => $srpm,
                        "path" => $altbase::path
                    },
                    iflist => {
                        "main" => (! defined($section))?1:0,
                        "spec" => ($section eq "spec")?1:0,
                        "changelog" => ($section eq "changelog")?1:0,
                        "docs" => ($section eq "docs")?1:0,
                        "bugs" => ($section eq "bugs" || $section eq "allbugs")?1:0,
                        "get" => ($section eq "get")?1:0
                    }
                );
            
    } else {
        print $altbase::header;
        print "internal error [stats.pl:70]";
        exit;
    }

    $sth->finish();
}

# cookies
my $session = cookie('ses');
if (defined($session) && length($session) == 15 && $session =~ /^\w+$/) {
    my $selectName_Q = "SELECT nick, uid FROM users WHERE session=? AND \
        (UNIX_TIMESTAMP(NOW()) - UNIX_TIMESTAMP(time)) < ?";
    $sth = $dbh->prepare($selectName_Q);
    $sth->execute($session,$altbase::TTL);
    @row = $sth-> fetchrow_array;
    if (@row) {
        my $selPref_Q = "SELECT type from userprefs WHERE uid=? AND rpm=?";
        $sth = $dbh->prepare($selPref_Q);
        $sth->execute($row[1],$srpm);
        $row = $sth-> fetchrow_array;
        
        $text .= $alt->parser(
                    HTML => "userpref",
                    vars => {
                        "nick" => tagClean($row[0]),
                        "package" => tagClean($srpm),
                        "bookStatus" => ($row eq "bookmark" || $row eq "all")?" checked":"",
                        "checkStatus" => ($row eq "pref" || $row eq "all")?" checked":""
                    },
                    iflist => {}
        );
        if ($row){
	    my $selSrpmInfo_Q = "SELECT CONCAT(srpm.version,'-',srpm.release), count(bug_id) FROM srpm, \
	    rpm LEFT JOIN bugs ON bugs.rpm=rpm.name WHERE srpm.name=? AND rpm.srpm=srpm.name \
	    GROUP BY srpm.name";

            $sth = $dbh->prepare($selSrpmInfo_Q);
            $sth->execute($srpm);
            my ($newv,$newb) = $sth->fetchrow_array;
            my $updatePrefs_Q = "UPDATE userprefs SET version=?, bugscount=? WHERE uid=? AND rpm=?";
            $sth = $dbh->prepare($updatePrefs_Q);
            $sth->execute($newv,$newb,$row[1],$srpm);
            $sth->finish();
        }
    }
}

$dbh->disconnect();
print $altbase::header . $text;

sub allMan {
    my $srpm = shift;
    my $alt = new altbase;
    my (@allman, $row, $mail, $text, $sth, @row);
    my $selAllMan_Q = "SELECT name FROM changelogs WHERE rpm=? ORDER BY time DESC";

    $text = "";

    $sth = $dbh->prepare($selAllMan_Q);
    $sth->execute($srpm);

    while ($row = $sth->fetchrow_array()){
        ($mail) = ($row =~ /<\s*([\w|\-]+)\s*[\@|at]/);
        if (isInArray($mail,\@allman)){
            next;
        }
        push(@allman,$mail);
    }

    my $selManInfo_Q = "SELECT packager, type FROM maintainers WHERE mail=?";
    $sth = $dbh->prepare($selManInfo_Q);

    foreach my $man (@allman){
        $sth->execute($man);

        if (@row = $sth->fetchrow_array()){
            $text .= $alt->parser(
                        HTML => "packager",
                        vars => {
                            "path" => $altbase::path,
                            "mail" => $man,
                            "packager" => $row[0]
                        },
                        iflist => {
                            "team" => ($row[1] eq "team")?0:1
                        }
            );
        }
    }
    $sth->finish();
    
    return $text;
}

sub allRpm {
    my $srpm = shift;
    my ($sth,$row,$text);
    my $alt = new altbase;
    my $selRPMS_Q = "SELECT name from rpm WHERE srpm=? ORDER BY name ASC";

    $sth = $dbh->prepare($selRPMS_Q);
    $sth->execute($srpm);
    while ( $row = $sth->fetchrow_array() ){
        $text .= $alt->parser(
                    HTML => "rpm",
                    vars => { "name" => $row },
                    iflist => {}
        );
    }

    return $text;
}

sub sizeOf {
    my $size = shift;

    if ($size < 1024*10) {
        return $size." b.";    
    } elsif ($size < 1024*1024*2) {
        return sprintf("%.2f",$size/1024)." Kb";
    } else {
        return sprintf("%.2f",$size/(1024*1024))." Mb";
    }
}

sub getNavigator {
    my ($main, $grp, $srpm) = @_;
    my $alt = new altbase;
    my $text = "";
    my @list = ();
    my %nav=();
    my ($sth,$row,$i);
    
    my $selMainSrpms_Q = "SELECT name FROM srpm WHERE packager=? AND age>0 ORDER BY name ASC";
    $sth = $dbh->prepare($selMainSrpms_Q);
    $sth->execute($main);
    while ($row = $sth->fetchrow_array){
        push(@list,$row);
    }
    if (@list) {
        for ($i=0; $i<=$#list; $i++){
            if ($list[$i] eq $srpm) {
                last;
            }
        }
        if ($i > 0) {
            $nav{"prevmain"} = $list[$i-1];
        }
        if ($i < $#list) {
            $nav{"nextmain"} = $list[$i+1];
        }
    }
    
    @list = ();
    my $selGrpSrpms_Q = "SELECT name FROM srpm WHERE rpmgroup=? AND age>0 ORDER BY name ASC";
    $sth = $dbh->prepare($selGrpSrpms_Q);
    $sth->execute($grp);
    while ($row = $sth->fetchrow_array){
        push(@list,$row);
    }
    if (@list) {
        for ($i=0; $i<=$#list; $i++){
            if ($list[$i] eq $srpm) {
                last;
            }
        }
        if ($i > 0) {
            $nav{"prevgrp"} = $list[$i-1];
        }
        if ($i < $#list) {
            $nav{"nextgrp"} = $list[$i+1];
        }
    }

    $text = $alt->parser(
                 HTML => "navigator",
                 vars => {
                     "path" => $altbase::path,
                     "prevmain" => $nav{"prevmain"},
                     "nextmain" => $nav{"nextmain"},
                     "prevgrp" => $nav{"prevgrp"},
                     "nextgrp" => $nav{"nextgrp"}
                 },
                 iflist => {
                     "prevmain" => ($nav{"prevmain"})?1:0,
                     "nextmain" => ($nav{"nextmain"})?1:0,
                     "prevgrp" => ($nav{"prevgrp"})?1:0,
                     "nextgrp" => ($nav{"nextgrp"})?1:0
                 }
    );
    
    return $text;
}